@@ -1,10 +1,13 @@
 from starwhale.core.dataset.type import (
+    Line,
     Link,
     Text,
     Audio,
     Image,
+    Point,
     Video,
     Binary,
+    Polygon,
     LinkAuth,
     LinkType,
     MIMEType,
@@ -37,6 +40,9 @@ __all__ = [
     "UserRawDataLoader",
     "Binary",
     "Text",
+    "Line",
+    "Point",
+    "Polygon",
     "Audio",
     "Video",
     "Image",
