@@ -16,10 +16,20 @@ from binascii import crc32
 import jsonlines
 from loguru import logger
 
-from starwhale.consts import AUTH_ENV_FNAME, DEFAULT_PROJECT, SWDS_DATA_FNAME_FMT
+from starwhale.consts import (
+    AUTH_ENV_FNAME,
+    DEFAULT_PROJECT,
+    STANDALONE_INSTANCE,
+    SWDS_DATA_FNAME_FMT,
+)
 from starwhale.base.uri import URI
 from starwhale.utils.fs import empty_dir, ensure_dir
-from starwhale.base.type import DataFormatType, DataOriginType, ObjectStoreType
+from starwhale.base.type import (
+    InstanceType,
+    DataFormatType,
+    DataOriginType,
+    ObjectStoreType,
+)
 from starwhale.utils.error import FormatError, NoSupportError
 from starwhale.core.dataset import model
 from starwhale.core.dataset.type import (
@@ -61,6 +71,7 @@ class BaseBuildExecutor(metaclass=ABCMeta):
         append_from_version: str = "",
         append_from_uri: t.Optional[URI] = None,
         data_mime_type: MIMEType = MIMEType.UNDEFINED,
+        instance_name: str = STANDALONE_INSTANCE,
     ) -> None:
         # TODO: add more docstring for args
         # TODO: validate group upper and lower?
@@ -80,11 +91,20 @@ class BaseBuildExecutor(metaclass=ABCMeta):
         self.dataset_name = dataset_name
         self.dataset_version = dataset_version
         self.tabular_dataset = TabularDataset(
-            dataset_name, dataset_version, project_name
+            dataset_name,
+            dataset_version,
+            project_name,
+            instance_name=instance_name,
         )
 
         self._forked_summary: t.Optional[DatasetSummary]
         if append and append_from_uri:
+            # TODO： controller supports cloud dataset fork api
+            if append_from_uri.instance_type == InstanceType.CLOUD:
+                raise NoSupportError(
+                    f"Can't build dataset from existed cloud dataset: {append_from_uri}"
+                )
+
             self._forked_last_seq_id, self._forked_rows = self.tabular_dataset.fork(
                 append_from_version
             )
@@ -125,6 +145,9 @@ class BaseBuildExecutor(metaclass=ABCMeta):
     def flush(self) -> None:
         self.tabular_dataset.flush()
 
+    def get_info(self) -> t.Optional[t.Dict[str, t.Any]]:
+        return None
+
     @abstractmethod
     def iter_item(self) -> t.Generator[t.Tuple, None, None]:
         raise NotImplementedError
@@ -305,6 +328,7 @@ class SWDSBinBuildExecutor(BaseBuildExecutor):
             print(f"data write close exception: {e}")
 
         self._copy_files(ds_copy_candidates)
+        self.tabular_dataset.info = self.get_info()  # type: ignore
 
         summary = DatasetSummary(
             rows=increased_rows,
@@ -445,6 +469,7 @@ class UserRawBuildExecutor(BaseBuildExecutor):
 
         self._copy_files(map_path_sign)
         self._copy_auth(auth_candidates)
+        self.tabular_dataset.info = self.get_info()  # type: ignore
 
         # TODO: provide fine-grained rows/increased rows by dataset pythonic api
         summary = DatasetSummary(
@@ -546,6 +571,7 @@ class RowWriter(threading.Thread):
         append_from_version: str = "",
         append_from_uri: t.Optional[URI] = None,
         append_with_swds_bin: bool = True,
+        instance_name: str = STANDALONE_INSTANCE,
     ) -> None:
         super().__init__(
             name=f"RowWriter-{dataset_name}-{dataset_version}-{project_name}"
@@ -561,6 +587,7 @@ class RowWriter(threading.Thread):
             "append": append,
             "append_from_version": append_from_version,
             "append_from_uri": append_from_uri,
+            "instance_name": instance_name,
         }
 
         self._queue: queue.Queue[t.Optional[DataRow]] = queue.Queue()
@@ -569,7 +596,7 @@ class RowWriter(threading.Thread):
 
         self._run_exception: t.Optional[Exception] = None
 
-        self.setDaemon(True)
+        self.daemon = True
         self._builder: t.Optional[BaseBuildExecutor] = None
         if append and append_from_version:
             _cls = create_generic_cls_by_mode(append_with_swds_bin, self.__iter__)
