@@ -6,12 +6,20 @@ from http import HTTPStatus
 from types import TracebackType
 from pathlib import Path
 from functools import wraps
+from itertools import islice
+from contextlib import ExitStack
 
 from loguru import logger
 
 from starwhale.utils import gen_uniq_version
-from starwhale.consts import HTTPMethod, DEFAULT_PAGE_IDX, DEFAULT_PAGE_SIZE
+from starwhale.consts import (
+    HTTPMethod,
+    DEFAULT_PAGE_IDX,
+    DEFAULT_PAGE_SIZE,
+    STANDALONE_INSTANCE,
+)
 from starwhale.base.uri import URI, URIType
+from starwhale.utils.fs import move_dir, empty_dir
 from starwhale.base.type import InstanceType
 from starwhale.base.cloud import CloudRequestMixed
 from starwhale.utils.error import ExistedError, NotFoundError, NoSupportError
@@ -19,7 +27,11 @@ from starwhale.core.dataset.type import DatasetConfig, DatasetSummary
 from starwhale.core.dataset.model import Dataset as CoreDataset
 from starwhale.core.dataset.model import StandaloneDataset
 from starwhale.core.dataset.store import DatasetStorage
+from starwhale.api._impl.data_store import TableEmptyException
 from starwhale.core.dataset.tabular import (
+    TabularDataset,
+    TabularDatasetInfo,
+    DatastoreWrapperDataset,
     get_dataset_consumption,
     DEFAULT_CONSUMPTION_BATCH_SIZE,
     TabularDatasetSessionConsumption,
@@ -33,6 +45,9 @@ _ItemType = t.Union[str, int, slice]
 _HandlerType = t.Optional[t.Union[t.Callable, BaseBuildExecutor]]
 _GItemType = t.Optional[t.Union[DataRow, t.List[DataRow]]]
 
+_DEFAULT_LOADER_WORKERS = 2
+_DEFAULT_LOADER_CACHE_SIZE = 20
+
 
 class _Tags:
     def __init__(self, core_dataset: CoreDataset) -> None:
@@ -102,8 +117,15 @@ class Dataset:
 
         self._append_use_swds_bin = False
         _summary = None
-        if self._check_uri_exists(_origin_uri):
+        origin_uri_exists = self._check_uri_exists(_origin_uri)
+        if origin_uri_exists:
             if create:
+                # TODO: support build cloud dataset from the existed dataset
+                if self.project_uri.instance_type == InstanceType.CLOUD:
+                    raise NoSupportError(
+                        f"Can't build dataset from the existed cloud dataset uri:{_origin_uri}"
+                    )
+
                 self._append_from_version = version
                 self._create_by_append = True
                 self._fork_dataset()
@@ -122,7 +144,14 @@ class Dataset:
             else:
                 raise ExistedError(f"{self.uri} was not found fo load")
 
-        self._summary = _summary or self.__core_dataset.summary()
+        self._summary: t.Optional[DatasetSummary]
+        if _summary:
+            self._summary = _summary
+        else:
+            if origin_uri_exists:
+                self._summary = self.__core_dataset.summary()
+            else:
+                self._summary = DatasetSummary()
 
         self._rows_cnt = self._summary.rows if self._summary else 0
         self._consumption: t.Optional[TabularDatasetSessionConsumption] = None
@@ -133,16 +162,32 @@ class Dataset:
         self._trigger_icode_build = False
         self._writer_lock = threading.Lock()
         self._row_writer: t.Optional[RowWriter] = None
-        self.__keys_cache: t.Set[t.Union[int, str]] = set()
         self._enable_copy_src = False
+        self._info_lock = threading.Lock()
+        self._info_ds_wrapper: t.Optional[DatastoreWrapperDataset] = None
+        self.__info: t.Optional[TabularDatasetInfo] = None
+
+        self._loader_cache_size = _DEFAULT_LOADER_CACHE_SIZE
+        self._loader_num_workers = _DEFAULT_LOADER_WORKERS
 
     def _fork_dataset(self) -> None:
         # TODO: support cloud dataset prepare in the tmp dir
         # TODO: lazy fork dataset
-        self.__core_dataset._prepare_snapshot()
-        self.__core_dataset._fork_swds(
-            self._create_by_append, self._append_from_version
-        )
+        if not isinstance(self.__core_dataset, StandaloneDataset):
+            raise NoSupportError(
+                f"only support standalone dataset fork: {self.__core_dataset}"
+            )
+
+        def _when_exit() -> None:
+            self.__core_dataset.store.building = False
+
+        with ExitStack() as stack:
+            stack.callback(_when_exit)
+            self.__core_dataset.store.building = True
+            self.__core_dataset._prepare_snapshot()
+            self.__core_dataset._fork_swds(
+                self._create_by_append, self._append_from_version
+            )
 
     def _auto_complete_version(self, version: str) -> str:
         version = version.strip()
@@ -205,6 +250,27 @@ class Dataset:
             )
         return self
 
+    def _clear_data_loader(self) -> None:
+        with self._lock:
+            self.__data_loaders = {}
+
+    def with_loader_config(
+        self, num_workers: t.Optional[int] = None, cache_size: t.Optional[int] = None
+    ) -> Dataset:
+        if len(self.__data_loaders) != 0:
+            raise RuntimeError(
+                f"loaders({list(self.__data_loaders)}) have already been initialized"
+            )
+
+        with self._lock:
+            if num_workers is not None:
+                self._loader_num_workers = num_workers
+
+            if cache_size is not None:
+                self._loader_cache_size = cache_size
+
+        return self
+
     def _get_data_loader(
         self, recreate: bool = False, disable_consumption: bool = False
     ) -> DataLoader:
@@ -218,11 +284,27 @@ class Dataset:
                 else:
                     consumption = self._consumption
 
-                _loader = get_data_loader(self.uri, session_consumption=consumption)
+                _loader = get_data_loader(
+                    self.uri,
+                    session_consumption=consumption,
+                    cache_size=self._loader_cache_size,
+                    num_workers=self._loader_num_workers,
+                )
                 self.__data_loaders[key] = _loader
 
         return _loader
 
+    def batch_iter(
+        self, batch_size: int = 1, drop_not_full: bool = False
+    ) -> t.Iterator[t.List[DataRow]]:
+        """Batch data into lists of length n. The last batch may be shorter."""
+        it = self.__iter__()
+        while True:
+            batch_data = list(islice(it, batch_size))
+            if not batch_data or (drop_not_full and len(batch_data) < batch_size):
+                return
+            yield batch_data
+
     def __iter__(self) -> t.Iterator[DataRow]:
         for row in self._get_data_loader():
             yield row
@@ -270,10 +352,9 @@ class Dataset:
 
         try:
             return _run()
-        except RuntimeError as e:
-            if str(e).startswith("table is empty"):
-                return None
-            raise
+        except TableEmptyException:
+            self._clear_data_loader()
+            return None
         except StopIteration:
             return None
 
@@ -340,7 +421,7 @@ class Dataset:
         if uri.instance_type == InstanceType.CLOUD:
             crm = CloudRequestMixed()
             ok, _ = crm.do_http_request_simple_ret(
-                path=f"/project/{uri.project}/{URIType.DATASET}/{uri.object.name}/version/{uri.object.version}/file",
+                path=f"/project/{uri.project}/{URIType.DATASET}/{uri.object.name}/version/{uri.object.version}",
                 method=HTTPMethod.HEAD,
                 instance_uri=uri,
                 ignore_status_codes=[HTTPStatus.NOT_FOUND],
@@ -353,11 +434,21 @@ class Dataset:
     def exists(self) -> bool:
         return self._check_uri_exists(self.uri)
 
+    @property
+    def info(self) -> TabularDatasetInfo:
+        if self.__info is not None:
+            return self.__info
+
+        with self._info_lock:
+            self._info_ds_wrapper = TabularDataset.from_uri(self.uri)._info_ds_wrapper
+            self.__info = TabularDatasetInfo.load_from_datastore(self._info_ds_wrapper)
+
+        return self.__info
+
     @_check_readonly
     def flush(self) -> None:
         loader = self._get_data_loader(disable_consumption=True)
         loader.tabular_dataset.flush()
-        self.__keys_cache = set()
 
         if self._row_writer:
             self._row_writer.flush()
@@ -384,7 +475,6 @@ class Dataset:
         return self.__core_dataset.history()
 
     def close(self) -> None:
-        self.__keys_cache = set()
         for _loader in self.__data_loaders.values():
             if not _loader:
                 continue  # pragma: no cover
@@ -399,18 +489,40 @@ class Dataset:
     def diff(self, cmp: Dataset) -> t.Dict:
         return self.__core_dataset.diff(cmp.uri)
 
-    def info(self) -> t.Dict[str, t.Any]:
+    def manifest(self) -> t.Dict[str, t.Any]:
         return self.__core_dataset.info()
 
     def head(self, n: int = 3, show_raw_data: bool = False) -> t.List[t.Dict]:
         # TODO: render artifact in JupyterNotebook
         return self.__core_dataset.head(n, show_raw_data)
 
-    def to_pytorch(self) -> t.Any:
-        raise NotImplementedError
+    def fetch_one(self, skip_fetch_data: bool = False) -> DataRow:
+        loader = self._get_data_loader(disable_consumption=True)
+        row = next(loader.tabular_dataset.scan())
+        return loader._unpack_row(row, skip_fetch_data)
 
-    def to_tensorflow(self) -> t.Any:
-        raise NotImplementedError
+    def to_pytorch(
+        self,
+        transform: t.Optional[t.Callable] = None,
+        drop_index: bool = True,
+        skip_default_transform: bool = False,
+    ) -> t.Any:
+        from starwhale.integrations.pytorch import TorchIterableDataset
+
+        return TorchIterableDataset(
+            dataset=self,
+            transform=transform,
+            drop_index=drop_index,
+            skip_default_transform=skip_default_transform,
+        )
+
+    def to_tensorflow(
+        self,
+        drop_index: bool = True,
+    ) -> t.Any:
+        from starwhale.integrations.tensorflow import to_tf_dataset
+
+        return to_tf_dataset(dataset=self, drop_index=drop_index)
 
     @_check_readonly
     @_forbid_handler_build
@@ -419,11 +531,6 @@ class Dataset:
     ) -> None:
         # TODO: tune the performance of getitem by cache
         self._trigger_icode_build = True
-        if not isinstance(self.__core_dataset, StandaloneDataset):
-            raise NoSupportError(
-                f"setitem only supports for standalone dataset: {self.__core_dataset}"
-            )
-
         _row_writer = self._get_row_writer()
 
         if not isinstance(key, (int, str)):
@@ -444,12 +551,8 @@ class Dataset:
         else:
             raise TypeError(f"value only supports tuple or DataRow type: {value}")
 
-        if key not in self.__keys_cache:
-            self.__keys_cache.add(key)
-            _item = self._getitem(key, skip_fetch_data=True)
-            if _item is None or len(_item) == 0:
-                self._rows_cnt += 1
-
+        # TODO improve accuracy of _rows_cnt during building
+        self._rows_cnt += 1
         _row_writer.update(row)
 
     def _get_row_writer(self) -> RowWriter:
@@ -475,20 +578,16 @@ class Dataset:
                     append_from_version = ""
 
                 # TODO: support alignment_bytes_size, volume_bytes_size arguments
-                if not isinstance(self.__core_dataset, StandaloneDataset):
-                    raise NoSupportError(
-                        f"setitem only supports for standalone dataset: {self.__core_dataset}"
-                    )
-
                 self._row_writer = RowWriter(
                     dataset_name=self.name,
                     dataset_version=self.version,
                     project_name=self.project_uri.project,
-                    workdir=self.__core_dataset.store.snapshot_workdir,  # TODO: use tmpdir which is same as dataset build command
+                    workdir=self.__core_dataset.store.tmp_dir,
                     append=self._create_by_append,
                     append_from_version=append_from_version,
                     append_from_uri=append_from_uri,
                     append_with_swds_bin=self._append_use_swds_bin,
+                    instance_name=self.project_uri.instance,
                 )
         return self._row_writer
 
@@ -513,8 +612,6 @@ class Dataset:
         for item in items:
             if not item or not isinstance(item, DataRow):
                 continue  # pragma: no cover
-            if item.index in self.__keys_cache:
-                self.__keys_cache.remove(item.index)
             loader.tabular_dataset.delete(item.index)
             self._rows_cnt -= 1
 
@@ -561,28 +658,76 @@ class Dataset:
     @_check_readonly
     @_forbid_handler_build
     def _do_build_from_interactive_code(self) -> None:
-        ds = self.__core_dataset
-        if isinstance(ds, StandaloneDataset):
-            if self._row_writer is None:
-                raise RuntimeError("row writer is none, no data was written")
+        if self._row_writer is None:
+            raise RuntimeError("row writer is none, no data was written")
 
-            self.flush()
-            self._row_writer.close()
-            # TODO: use the elegant method to refactor manifest update
-            self._summary = self._row_writer.summary
-            self._summary.rows = len(self)
-            ds._manifest["dataset_summary"] = self._summary.asdict()
-            ds._calculate_signature()
-            ds._render_manifest()
-            ds._make_swds_meta_tar()
-            ds._make_auto_tags()
+        if self.__info is not None and self._info_ds_wrapper is not None:
+            self.__info.save_to_datastore(self._info_ds_wrapper)
+
+        self.flush()
+        self._row_writer.close()
+        self._summary = self._row_writer.summary
+
+        # TODO: add len api for tabular_dataset to reduce overhead here
+        if self._row_writer._builder:
+            table_rows = [
+                row for row in self._row_writer._builder.tabular_dataset.scan()
+            ]
+            self._summary.rows = len(table_rows)
+
+        if isinstance(self.__core_dataset, StandaloneDataset):
+            local_ds = self.__core_dataset
+            local_uri = self.uri
         else:
-            # TODO: support cloud dataset build
-            raise NoSupportError("only support standalone dataset build")
+            local_uri = URI.capsulate_uri(
+                instance=STANDALONE_INSTANCE,
+                project=self.uri.project,
+                obj_type=self.uri.object.typ,
+                obj_name=self.uri.object.name,
+                obj_ver=self.uri.object.version,
+            )
+            local_ds = StandaloneDataset(local_uri)
+            local_ds.store._tmp_dir = self.__core_dataset.store.tmp_dir
+            setattr(local_ds, "_version", self.version)
+
+        def _when_standalone_exit() -> None:
+            local_ds._make_auto_tags()
+            move_dir(local_ds.store.tmp_dir, local_ds.store.snapshot_workdir)
+
+        def _when_cloud_exit() -> None:
+            from starwhale.core.dataset.copy import DatasetCopy
+
+            dc = DatasetCopy(
+                str(local_uri), str(self.uri), URIType.DATASET
+            ).with_disable_datastore()
+            dc._do_upload_bundle_dir(workdir=local_ds.store.tmp_dir)
+            empty_dir(local_ds.store.tmp_dir)
+
+        def _when_exit() -> None:
+            local_ds.store.building = False
+            if isinstance(self.__core_dataset, StandaloneDataset):
+                _when_standalone_exit()
+            else:
+                _when_cloud_exit()
+
+        with ExitStack() as stack:
+            stack.callback(_when_exit)
+            local_ds.store.building = True
+            local_ds._manifest["dataset_summary"] = self._summary.asdict()
+            local_ds._calculate_signature()
+            local_ds._render_manifest()
+            local_ds._make_swds_meta_tar()
 
     @_check_readonly
     @_forbid_icode_build
     def _do_build_from_handler(self) -> None:
+        # TODO: support build dataset for cloud uri directly
+        if self.project_uri.instance_type == InstanceType.CLOUD:
+            raise NoSupportError("no support to build cloud dataset directly")
+
+        if self.__info is not None and self._info_ds_wrapper is not None:
+            self.__info.save_to_datastore(self._info_ds_wrapper)
+
         self._trigger_icode_build = True
         config = DatasetConfig(
             name=self.name,
@@ -608,10 +753,6 @@ class Dataset:
 
     @_check_readonly
     def build(self) -> None:
-        # TODO: support build dataset for cloud uri directly
-        if self.project_uri.instance_type == InstanceType.CLOUD:
-            raise NoSupportError("no support to build cloud dataset directly")
-
         if self._trigger_icode_build:
             self._do_build_from_interactive_code()
         elif self._trigger_handler_build and self.build_handler:
