@@ -1,11 +1,14 @@
 from ._impl.dataset import (
+    Line,
     Link,
     Text,
     Audio,
     Image,
+    Point,
     Video,
     Binary,
     Dataset,
+    Polygon,
     LinkAuth,
     LinkType,
     MIMEType,
@@ -43,6 +46,9 @@ __all__ = [
     "UserRawDataLoader",
     "Binary",
     "Text",
+    "Line",
+    "Point",
+    "Polygon",
     "Audio",
     "Video",
     "Image",
