@@ -1,3 +1,5 @@
+from enum import Enum, unique
+
 from starwhale.utils.error import NoSupportError
 
 
@@ -12,7 +14,7 @@ class URIType:
     MODEL = "model"
     DATASET = "dataset"
     RUNTIME = "runtime"
-    JOB = "job"
+    EVALUATION = "evaluation"
     UNKNOWN = "unknown"
 
 
@@ -22,18 +24,16 @@ class JobType:
 
 class EvalTaskType:
     ALL = "all"
-    PPL = "ppl"
-    CMP = "cmp"
+    SINGLE = "single"
 
 
 class RunSubDirType:
-    RESULT = "result"
     DATASET = "dataset"
-    PPL_RESULT = "ppl_result"
     STATUS = "status"
+    RUNLOG = "runlog"
     LOG = "log"
     SWMP = "swmp"
-    CONFIG = "config"
+    SWRT = "swrt"
 
 
 class JobOperationType:
@@ -52,6 +52,18 @@ class BundleType:
     RUNTIME = ".swrt"
 
 
+class RuntimeArtifactType:
+    RUNTIME = "runtime_yaml"
+    DEPEND = "dependencies"
+    WHEELS = "wheels"
+    FILES = "files"
+
+
+class RuntimeLockFileType:
+    VENV = "requirements-sw-lock.txt"
+    CONDA = "conda-sw-lock.yaml"
+
+
 def get_bundle_type_by_uri(uri_type: str) -> str:
     if uri_type == URIType.DATASET:
         return BundleType.DATASET
@@ -61,3 +73,33 @@ def get_bundle_type_by_uri(uri_type: str) -> str:
         return BundleType.RUNTIME
     else:
         raise NoSupportError(uri_type)
+
+
+@unique
+class DataFormatType(Enum):
+    SWDS_BIN = "swds_bin"
+    USER_RAW = "user_raw"
+    UNDEFINED = "undefined"
+
+
+@unique
+class ObjectStoreType(Enum):
+    LOCAL = "local"
+    REMOTE = "remote"
+    UNDEFINED = "undefined"
+
+
+@unique
+class DataOriginType(Enum):
+    NEW = "+"
+    INHERIT = "~"
+
+
+@unique
+class DependencyType(Enum):
+    PIP_PKG = "pip_pkg"
+    PIP_REQ_FILE = "pip_req_file"
+    CONDA_PKG = "conda_pkg"
+    CONDA_ENV_FILE = "conda_env_file"
+    WHEEL = "wheel"
+    NATIVE_FILE = "native_file"
