@@ -1,4 +1,6 @@
 import pathlib
+from enum import Enum, unique
+from dataclasses import dataclass
 
 # TODO: use str path, not Path Class
 HOMEDIR = pathlib.Path.home()
@@ -17,7 +19,7 @@ DEFAULT_STARWHALE_API_VERSION = "1.0"
 DEFAULT_MANIFEST_NAME = "_manifest.yaml"
 DEFAULT_EVALUATION_JOB_NAME = "default"
 DEFAULT_EVALUATION_JOBS_FNAME = "eval_jobs.yaml"
-DEFAULT_EVALUATION_SVC_META_FNAME = "svc.yaml"
+DEFAULT_EVALUATION_SVC_META_FNAME = "svc.json"
 DEFAULT_EVALUATION_PIPELINE = "starwhale.core.model.default_handler"
 DEFAULT_LOCAL_SW_CONTROLLER_ADDR = "localhost:7827"
 LOCAL_CONFIG_VERSION = "2.0"
@@ -93,6 +95,7 @@ class SWDSBackendType:
     S3 = "s3"
     LocalFS = "local_fs"
     SignedUrl = "signed_url"
+    Http = "http"
 
 
 class EvalHandlerType:
@@ -116,8 +119,35 @@ class SWDSSubFileType:
     META = "swds_meta"
 
 
+@unique
+class FileDesc(Enum):
+    MANIFEST = "MANIFEST"
+    SRC = "SRC"
+    SRC_TAR = "SRC_TAR"
+    MODEL = "MODEL"
+    DATA = "DATA"
+
+
+class FileFlag:
+    UNCHANGED = "unchanged"
+    ADDED = "added"
+    UPDATED = "updated"
+    DELETED = "deleted"
+
+
+@dataclass
+class FileNode:
+    path: pathlib.Path
+    name: str
+    size: int
+    file_desc: FileDesc
+    signature: str = ""
+    flag: str = ""
+
+
 SWDS_DATA_FNAME_FMT = "data_ubyte_{index}.%s" % SWDSSubFileType.BIN
 ARCHIVED_SWDS_META_FNAME = "archive.%s" % SWDSSubFileType.META
+SWMP_SRC_FNAME = "src.tar"
 
 CURRENT_FNAME = "current"
 
