@@ -6,6 +6,7 @@ import sys
 import json
 import shutil
 import typing as t
+import threading
 from abc import ABCMeta, abstractmethod
 from types import TracebackType
 from pathlib import Path
@@ -110,7 +111,7 @@ class DatasetStorage(BaseStorage):
             raise NotFoundError(f"data origin file: {src}")
 
         if not src.is_file():
-            raise NoSupportError(f"{src} is not file type")
+            raise NoSupportError(f"{src} is not file")
 
         sign_name = blake2b_file(src)
         dest = cls._get_object_store_path(sign_name)
@@ -297,6 +298,8 @@ class ObjectStore:
             self.backend = S3StorageBackend(conn)
         elif backend == SWDSBackendType.SignedUrl:
             self.backend = SignedUrlBackend(dataset_uri)
+        elif backend == SWDSBackendType.Http:
+            self.backend = HttpBackend()
         else:
             self.backend = LocalFSStorageBackend()
 
@@ -314,10 +317,14 @@ class ObjectStore:
             raise FieldTypeOrValueError("data_link is empty")
 
         # TODO: support other uri type
-        if data_link.uri.startswith("s3://"):
+        if data_link.scheme in ("s3", "minio", "oss", "aliyun"):
             backend = SWDSBackendType.S3
             conn = S3Connection.from_uri(data_link.uri, auth_name)
             bucket = conn.bucket
+        elif data_link.scheme in ["http", "https"]:
+            backend = SWDSBackendType.Http
+            bucket = ""
+            conn = None
         else:
             backend = SWDSBackendType.LocalFS
             bucket = ""
@@ -361,29 +368,32 @@ class StorageBackend(metaclass=ABCMeta):
 
 
 class S3StorageBackend(StorageBackend):
+
+    lock_s3_creation = threading.Lock()
+
     def __init__(
         self,
         conn: S3Connection,
     ):
         super().__init__(kind=SWDSBackendType.S3)
-
-        self.s3 = boto3.resource(
-            "s3",
-            endpoint_url=conn.endpoint,
-            aws_access_key_id=conn.access_key,
-            aws_secret_access_key=conn.secret_key,
-            config=S3Config(
-                s3=conn.extra_s3_configs,
-                connect_timeout=conn.connect_timeout,
-                read_timeout=conn.read_timeout,
-                signature_version="s3v4",
-                retries={
-                    "total_max_attempts": conn.total_max_attempts,
-                    "mode": "standard",
-                },
-            ),
-            region_name=conn.region,
-        )
+        with S3StorageBackend.lock_s3_creation:
+            self.s3 = boto3.resource(
+                "s3",
+                endpoint_url=conn.endpoint,
+                aws_access_key_id=conn.access_key,
+                aws_secret_access_key=conn.secret_key,
+                config=S3Config(
+                    s3=conn.extra_s3_configs,
+                    connect_timeout=conn.connect_timeout,
+                    read_timeout=conn.read_timeout,
+                    signature_version="s3v4",
+                    retries={
+                        "total_max_attempts": conn.total_max_attempts,
+                        "mode": "standard",
+                    },
+                ),
+                region_name=conn.region,
+            )
 
     def _make_file(
         self, bucket: str, key_compose: t.Tuple[Link, int, int]
@@ -453,6 +463,22 @@ class SignedUrlBackend(StorageBackend, CloudRequestMixed):
         return r["data"].get(uri, "")  # type: ignore
 
 
+class HttpBackend(StorageBackend, CloudRequestMixed):
+    def __init__(self) -> None:
+        super().__init__(kind=SWDSBackendType.Http)
+
+    @http_retry
+    def _make_file(
+        self, auth: str, key_compose: t.Tuple[Link, int, int]
+    ) -> FileLikeObj:
+        _key, _start, _end = key_compose
+        return HttpBufferedFileLike(
+            url=_key.uri,
+            headers={"Range": f"bytes={_start or 0}-{_end or sys.maxsize}"},
+            timeout=90,
+        )
+
+
 _BFType = t.TypeVar("_BFType", bound="BaseBufferedFileLike")
 
 
