@@ -10,8 +10,8 @@ from pathlib import Path
 from functools import partial
 from urllib.parse import urlparse
 
+import numpy
 import requests
-from numpy import ndarray
 
 from starwhale.utils import load_yaml, convert_to_bytes, validate_obj_name
 from starwhale.consts import (
@@ -23,8 +23,11 @@ from starwhale.base.uri import URI
 from starwhale.utils.fs import FilePosition
 from starwhale.base.type import URIType, InstanceType
 from starwhale.base.mixin import ASDictMixin
-from starwhale.api.service import Request
-from starwhale.utils.error import NoSupportError, FieldTypeOrValueError
+from starwhale.utils.error import (
+    NoSupportError,
+    FieldTypeOrValueError,
+    MissingDependencyError,
+)
 from starwhale.utils.retry import http_retry
 from starwhale.api._impl.data_store import SwObject, _TYPE_DICT
 
@@ -129,6 +132,7 @@ class MIMEType(Enum):
     GIF = "image/gif"
     APNG = "image/apng"
     AVIF = "image/avif"
+    PPM = "image/x-portable-pixmap"
     MP4 = "video/mp4"
     AVI = "video/avi"
     WEBM = "video/webm"
@@ -159,13 +163,14 @@ class MIMEType(Enum):
             ".mp3": cls.MP3,
             ".mp4": cls.MP4,
             ".avif": cls.AVIF,
+            ".ppm": cls.PPM,
             ".avi": cls.AVI,
             ".webm": cls.WEBM,
             ".wav": cls.WAV,
             ".csv": cls.CSV,
             ".txt": cls.PLAIN,
         }
-        return _map.get(Path(name).suffix, MIMEType.UNDEFINED)
+        return _map.get(Path(name).suffix.lower(), MIMEType.UNDEFINED)
 
 
 LocalFSLinkAuth = partial(LinkAuth, ltype=LinkType.LocalFS)
@@ -190,7 +195,7 @@ class ArtifactType(Enum):
 _TBAType = t.TypeVar("_TBAType", bound="BaseArtifact")
 
 
-class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
+class BaseArtifact(ASDictMixin, metaclass=ABCMeta):
     def __init__(
         self,
         fp: _TArtifactFP,
@@ -198,6 +203,7 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
         display_name: str = "",
         shape: t.Optional[_TShape] = None,
         mime_type: t.Optional[MIMEType] = None,
+        dtype: t.Any = numpy.int8,
         encoding: str = "",
     ) -> None:
         self.fp = str(fp) if isinstance(fp, Path) else fp
@@ -207,12 +213,17 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
         self.display_name = display_name or os.path.basename(_fpath)
         self._mime_type = (mime_type or MIMEType.create_by_file_suffix(_fpath)).value
         self.shape = list(shape) if shape else shape
+        self._dtype_name: str = numpy.dtype(dtype).name
         self.encoding = encoding
         self._do_validate()
 
     def _do_validate(self) -> None:
         ...
 
+    @property
+    def dtype(self) -> numpy.dtype:
+        return numpy.dtype(self._dtype_name)
+
     @property
     def type(self) -> ArtifactType:
         return ArtifactType(self._type)
@@ -255,7 +266,6 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
         else:
             raise NoSupportError(f"Artifact reflect error: {data_type}")
 
-    # TODO: add to_tensor, to_numpy method
     def to_bytes(self, encoding: str = "utf-8") -> bytes:
         if isinstance(self.fp, bytes):
             return self.fp
@@ -269,7 +279,7 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
         else:
             raise NoSupportError(f"read raw for type:{type(self.fp)}")
 
-    def to_numpy(self) -> ndarray:
+    def to_numpy(self) -> numpy.ndarray:
         raise NotImplementedError
 
     def to_json(self) -> str:
@@ -280,9 +290,6 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
 
     to_pt_tensor = to_tensor
 
-    def to_tf_tensor(self) -> t.Any:
-        ...
-
     def carry_raw_data(self: _TBAType) -> _TBAType:
         self._raw_base64_data = base64.b64encode(self.to_bytes()).decode()
         return self
@@ -304,27 +311,18 @@ class BaseArtifact(ASDictMixin, Request, metaclass=ABCMeta):
 
     __repr__ = __str__
 
-    def load(self, data: t.Union[str, bytes, t.Dict[str, t.Any]]) -> BaseArtifact:
-        if isinstance(data, bytes):
-            raw = data
-        elif isinstance(data, str):
-            raw = base64.b64decode(data)
-        elif isinstance(data, dict):
-            raw = base64.b64decode(data["data"])
-            return self.reflect(raw, data)
-        else:
-            raise NoSupportError(f"load raw for type:{type(data)}")
-        self.fp = raw
-        return self
-
 
 class Binary(BaseArtifact, SwObject):
     def __init__(
         self,
         fp: _TArtifactFP = b"",
         mime_type: MIMEType = MIMEType.UNDEFINED,
+        dtype: t.Type = numpy.bytes_,
     ) -> None:
-        super().__init__(fp, ArtifactType.Binary, "", (1,), mime_type)
+        super().__init__(fp, ArtifactType.Binary, "", (), mime_type, dtype=dtype)
+
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_bytes(), dtype=self.dtype)
 
 
 class Image(BaseArtifact, SwObject):
@@ -336,6 +334,7 @@ class Image(BaseArtifact, SwObject):
         mime_type: MIMEType = MIMEType.UNDEFINED,
         as_mask: bool = False,
         mask_uri: str = "",
+        dtype: t.Type = numpy.uint8,
     ) -> None:
         self.as_mask = as_mask
         self.mask_uri = mask_uri
@@ -345,6 +344,7 @@ class Image(BaseArtifact, SwObject):
             display_name=display_name,
             shape=shape or (None, None, 3),
             mime_type=mime_type,
+            dtype=dtype,
         )
 
     def _do_validate(self) -> None:
@@ -355,11 +355,30 @@ class Image(BaseArtifact, SwObject):
             MIMEType.SVG,
             MIMEType.GIF,
             MIMEType.APNG,
+            MIMEType.PPM,
             MIMEType.GRAYSCALE,
             MIMEType.UNDEFINED,
         ):
             raise NoSupportError(f"Image type: {self.mime_type}")
 
+    def to_pil(self) -> t.Any:
+        try:
+            from PIL import Image as PILImage
+        except ImportError:  # pragma: no cover
+            raise MissingDependencyError(
+                "pillow is required to convert Starwhale Image to Pillow Image, please install pillow with 'pip install pillow' or 'pip install starwhale[image]'."
+            )
+
+        return PILImage.open(io.BytesIO(self.to_bytes()))
+
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_pil())
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_numpy_to_tensor
+
+        return convert_numpy_to_tensor(self.to_numpy())
+
 
 class GrayscaleImage(Image):
     def __init__(
@@ -369,6 +388,7 @@ class GrayscaleImage(Image):
         shape: t.Optional[_TShape] = None,
         as_mask: bool = False,
         mask_uri: str = "",
+        dtype: t.Type = numpy.uint8,
     ) -> None:
         shape = shape or (None, None)
         super().__init__(
@@ -378,6 +398,7 @@ class GrayscaleImage(Image):
             mime_type=MIMEType.GRAYSCALE,
             as_mask=as_mask,
             mask_uri=mask_uri,
+            dtype=dtype,
         )
 
 
@@ -388,9 +409,12 @@ class Audio(BaseArtifact, SwObject):
         display_name: str = "",
         shape: t.Optional[_TShape] = None,
         mime_type: t.Optional[MIMEType] = None,
+        dtype: t.Type = numpy.float64,
     ) -> None:
         shape = shape or (None,)
-        super().__init__(fp, ArtifactType.Audio, display_name, shape, mime_type)
+        super().__init__(
+            fp, ArtifactType.Audio, display_name, shape, mime_type, dtype=dtype
+        )
 
     def _do_validate(self) -> None:
         if self.mime_type not in (
@@ -400,6 +424,24 @@ class Audio(BaseArtifact, SwObject):
         ):
             raise NoSupportError(f"Audio type: {self.mime_type}")
 
+    def to_numpy(self) -> numpy.ndarray:
+        try:
+            import soundfile
+        except ImportError:  # pragma: no cover
+            raise MissingDependencyError(
+                "soundfile is required to convert Starwhale Auto to numpy ndarray, please install soundfile with 'pip install soundfile' or 'pip install starwhale[audio]'."
+            )
+
+        array, _ = soundfile.read(
+            io.BytesIO(self.to_bytes()), dtype=self.dtype.name, always_2d=True
+        )
+        return array  # type: ignore[no-any-return]
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_numpy_to_tensor
+
+        return convert_numpy_to_tensor(self.to_numpy())
+
 
 class Video(BaseArtifact, SwObject):
     def __init__(
@@ -408,9 +450,18 @@ class Video(BaseArtifact, SwObject):
         display_name: str = "",
         shape: t.Optional[_TShape] = None,
         mime_type: t.Optional[MIMEType] = None,
+        dtype: t.Type = numpy.uint8,
     ) -> None:
         shape = shape or (None,)
-        super().__init__(fp, ArtifactType.Video, display_name, shape, mime_type)
+        super().__init__(
+            fp, ArtifactType.Video, display_name, shape, mime_type, dtype=dtype
+        )
+
+    # TODO： support to_tensor methods
+
+    def to_numpy(self) -> numpy.ndarray:
+        # TODO: support video encode/decode
+        return numpy.array(self.to_bytes(), dtype=self.dtype)
 
     def _do_validate(self) -> None:
         if self.mime_type not in (
@@ -453,15 +504,122 @@ class BoundingBox(ASDictMixin, SwObject):
         self.width = width
         self.height = height
 
+    @property
+    def shape(self) -> t.Tuple[int]:
+        return (4,)
+
+    @property
+    def dtype(self) -> numpy.dtype:
+        return numpy.dtype(numpy.float64)
+
     def to_list(self) -> t.List[float]:
         return [self.x, self.y, self.width, self.height]
 
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_list(), self.dtype)
+
+    def to_bytes(self) -> bytes:
+        return self.to_numpy().tobytes()
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_list_to_tensor
+
+        return convert_list_to_tensor(self.to_list())
+
     def __str__(self) -> str:
         return f"BoundingBox: point:({self.x}, {self.y}), width: {self.width}, height: {self.height})"
 
     __repr__ = __str__
 
 
+class Line(ASDictMixin, SwObject):
+    def __init__(self, points: t.List[Point]) -> None:
+        self._type = "line"
+        self.points = points
+
+    @property
+    def dtype(self) -> numpy.dtype:
+        return numpy.dtype(numpy.float64)
+
+    def to_list(self) -> t.List[t.List[float]]:
+        return [p.to_list() for p in self.points]
+
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_list(), self.dtype)
+
+    def to_bytes(self) -> bytes:
+        return self.to_numpy().tobytes()
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_list_to_tensor
+
+        return convert_list_to_tensor(self.to_list())
+
+    def __str__(self) -> str:
+        return f"Line: {self.to_list()}"
+
+    __repr__ = __str__
+
+
+class Point(ASDictMixin, SwObject):
+    def __init__(self, x: float = 0, y: float = 0) -> None:
+        self._type = "point"
+        self.x = x
+        self.y = y
+
+    @property
+    def dtype(self) -> numpy.dtype:
+        return numpy.dtype(numpy.float64)
+
+    def to_list(self) -> t.List[float]:
+        return [self.x, self.y]
+
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_list(), self.dtype)
+
+    def to_bytes(self) -> bytes:
+        return self.to_numpy().tobytes()
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_list_to_tensor
+
+        return convert_list_to_tensor(self.to_list())
+
+    def __str__(self) -> str:
+        return f"Point: ({self.x}, {self.y})"
+
+    __repr__ = __str__
+
+
+class Polygon(ASDictMixin, SwObject):
+    def __init__(self, points: t.List[Point]) -> None:
+        self._type = "polygon"
+        self.points = points
+
+    @property
+    def dtype(self) -> numpy.dtype:
+        return numpy.dtype(numpy.float64)
+
+    def to_list(self) -> t.List[t.List[float]]:
+        return [p.to_list() for p in self.points]
+
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_list(), self.dtype)
+
+    def to_bytes(self) -> bytes:
+        return self.to_numpy().tobytes()
+
+    def to_tensor(self) -> t.Any:
+        from starwhale.integrations.pytorch import convert_list_to_tensor
+
+        return convert_list_to_tensor(self.to_list())
+
+    def __str__(self) -> str:
+        return f"Polygon: {self.to_list()}"
+
+    __repr__ = __str__
+
+
 class Text(BaseArtifact, SwObject):
     DEFAULT_ENCODING = "utf-8"
 
@@ -472,18 +630,23 @@ class Text(BaseArtifact, SwObject):
             fp=b"",
             type=ArtifactType.Text,
             display_name=f"{content[:SHORT_VERSION_CNT]}...",
-            shape=(1,),
+            shape=(),
             mime_type=MIMEType.PLAIN,
             encoding=encoding,
+            dtype=numpy.str_,
         )
 
     def to_bytes(self, encoding: str = "") -> bytes:
         return self.content.encode(encoding or self.encoding)
 
+    def to_numpy(self) -> numpy.ndarray:
+        return numpy.array(self.to_str(), dtype=self.dtype)
+
     def to_str(self) -> str:
         return self.content
 
 
+# TODO: support tensorflow transform
 # https://cocodataset.org/#format-data
 class COCOObjectAnnotation(ASDictMixin, SwObject):
     def __init__(
@@ -538,6 +701,7 @@ class COCOObjectAnnotation(ASDictMixin, SwObject):
             )
 
 
+# TODO: support tensorflow transform
 class Link(ASDictMixin, SwObject):
     def __init__(
         self,
@@ -550,6 +714,8 @@ class Link(ASDictMixin, SwObject):
     ) -> None:
         self._type = "link"
         self.uri = (str(uri)).strip()
+        _up = urlparse(self.uri)
+        self.scheme = _up.scheme
         self.offset = offset
         self.size = size
         self.auth = auth
@@ -608,8 +774,7 @@ class Link(ASDictMixin, SwObject):
             key_compose = self, 0, 0
             store = ObjectStore.to_signed_http_backend(dataset_uri)
         else:
-            _up = urlparse(self.uri)
-            if _up.scheme:
+            if self.scheme:
                 key_compose = (
                     Link(self.local_fs_uri) if self.local_fs_uri else self,
                     0,
@@ -780,7 +945,7 @@ class JsonDict(SwObject):
     @classmethod
     def to_data(cls, d: t.Any) -> t.Any:
         """
-        unwrapp JsonDict to dict
+        unwrap JsonDict to dict
         """
         if isinstance(d, JsonDict):
             return d.asdict()
