@@ -2,6 +2,7 @@ import typing as t
 
 import click
 
+from starwhale import URI, URIType
 from starwhale.consts import DefaultYAMLName, DEFAULT_PAGE_IDX, DEFAULT_PAGE_SIZE
 from starwhale.utils.cli import AliasedGroup
 from starwhale.consts.env import SWEnv
@@ -104,6 +105,21 @@ def _info(view: t.Type[ModelTermView], model: str, fullname: bool) -> None:
     view(model).info(fullname)
 
 
+@model_cmd.command("diff", help="model version diff")
+@click.argument("base_uri", required=True)
+@click.argument("compare_uri", required=True)
+@click.option(
+    "--show-details",
+    is_flag=True,
+    help="Show different detail by the model package files",
+)
+@click.pass_obj
+def _diff(
+    view: t.Type[ModelTermView], base_uri: str, compare_uri: str, show_details: bool
+) -> None:
+    view(base_uri).diff(URI(compare_uri, expected_type=URIType.MODEL), show_details)
+
+
 @model_cmd.command("list", aliases=["ls"], help="List Model")
 @click.option("-p", "--project", default="", help="Project URI")
 @click.option("-f", "--fullname", is_flag=True, help="Show fullname of model version")
@@ -154,20 +170,6 @@ def _recover(model: str, force: bool) -> None:
     ModelTermView(model).recover(force)
 
 
-@model_cmd.command(
-    "extract", help="[ONLY Standalone]Extract local model bundle tar file into workdir"
-)
-@click.argument("model")
-@click.option("-f", "--force", is_flag=True, help="Force to extract model bundle")
-@click.option(
-    "--target-dir",
-    default="",
-    help="Extract target dir.if omitted, swcli will use starwhale default workdir",
-)
-def _extract(model: str, force: bool, target_dir: str) -> None:
-    ModelTermView(model).extract(force, target_dir)
-
-
 @model_cmd.command("eval")
 @click.argument("target")
 @click.option(
@@ -205,6 +207,17 @@ def _extract(model: str, force: bool, target_dir: str) -> None:
     multiple=True,
     help=f"dataset uri, env is {SWEnv.dataset_uri}",
 )
+@click.option(
+    "--use-docker",
+    is_flag=True,
+    help="[ONLY Standalone]use docker to run evaluation job",
+)
+@click.option("--gencmd", is_flag=True, help="[ONLY Standalone]gen docker run command")
+@click.option(
+    "--image",
+    default="",
+    help="[ONLY Standalone]the image used when use docker",
+)
 def _eval(
     project: str,
     target: str,
@@ -215,6 +228,9 @@ def _eval(
     task_index: int,
     override_task_num: int,
     runtime: str,
+    use_docker: bool,
+    gencmd: bool,
+    image: str,
 ) -> None:
     """
     [ONLY Standalone]Run evaluation processing with root dir of {target}.
@@ -231,30 +247,30 @@ def _eval(
         task_index=task_index,
         task_num=override_task_num,
         dataset_uris=datasets,
+        use_docker=use_docker,
+        gencmd=gencmd,
+        image=image,
     )
 
 
 @model_cmd.command("serve")
-@click.argument("target")
+@click.argument("target", required=False, default="")
 @click.option(
     "-f",
     "--model-yaml",
     default=DefaultYAMLName.MODEL,
     help="Model yaml filename, default use ${MODEL_DIR}/model.yaml file",
 )
-@click.option(
-    "--handlers",
-    default=None,
-    help="List of service handlers, use all by default",
-    multiple=True,
-)
+@click.option("-r", "--runtime", default="", help="runtime uri")
+@click.option("-m", "--model", default="", help="model uri")
 @click.option("--host", default="", help="The host to listen on")
 @click.option("--port", default=8080, help="The port of the server")
 def _serve(
     target: str,
     model_yaml: str,
+    runtime: str,
+    model: str,
     host: str,
     port: int,
-    handlers: t.Optional[t.List[str]],
 ) -> None:
-    ModelTermView.serve(target, model_yaml, host, port, handlers)
+    ModelTermView.serve(target, model_yaml, runtime, model, host, port)
