@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import os
+import copy
+import json
 import typing as t
 import tarfile
 from abc import ABCMeta
@@ -11,10 +13,16 @@ import yaml
 from fs import open_fs
 from loguru import logger
 from fs.copy import copy_fs, copy_file
+from fs.walk import Walker
 from fs.tarfs import TarFS
 
+from starwhale import PipelineHandler
 from starwhale.utils import console, now_str, load_yaml, gen_uniq_version
 from starwhale.consts import (
+    FileDesc,
+    FileFlag,
+    FileNode,
+    SWMP_SRC_FNAME,
     DefaultYAMLName,
     EvalHandlerType,
     DEFAULT_PAGE_IDX,
@@ -28,7 +36,13 @@ from starwhale.consts import (
 )
 from starwhale.base.tag import StandaloneTag
 from starwhale.base.uri import URI
-from starwhale.utils.fs import move_dir, ensure_dir, ensure_file
+from starwhale.utils.fs import (
+    move_dir,
+    file_stat,
+    ensure_dir,
+    ensure_file,
+    blake2b_file,
+)
 from starwhale.base.type import URIType, BundleType, InstanceType
 from starwhale.base.cloud import CloudRequestMixed, CloudBundleModelMixin
 from starwhale.base.mixin import ASDictMixin
@@ -41,7 +55,7 @@ from starwhale.api._impl.job import Parser, Context, context_holder
 from starwhale.core.job.model import STATUS, Generator
 from starwhale.utils.progress import run_with_progress_bar
 from starwhale.core.eval.store import EvaluationStorage
-from starwhale.base.bundle_copy import BundleCopy
+from starwhale.core.model.copy import ModelCopy
 from starwhale.core.model.store import ModelStorage
 from starwhale.core.job.scheduler import Scheduler
 
@@ -166,7 +180,7 @@ class Model(BaseBundle, metaclass=ABCMeta):
         force: bool = False,
         dest_local_project_uri: str = "",
     ) -> None:
-        bc = BundleCopy(
+        bc = ModelCopy(
             src_uri,
             dest_uri,
             URIType.MODEL,
@@ -175,6 +189,24 @@ class Model(BaseBundle, metaclass=ABCMeta):
         )
         bc.do()
 
+    def diff(self, compare_uri: URI) -> t.Dict[str, t.Any]:
+        raise NotImplementedError
+
+
+def resource_to_file_node(
+    files: t.List[t.Dict[str, t.Any]], parent_path: Path
+) -> t.Dict[str, FileNode]:
+    return {
+        _f["path"]: FileNode(
+            path=parent_path / _f["path"],
+            name=_f.get("name") or os.path.basename(_f["path"]),
+            size=_f.get("size") or file_stat(parent_path / _f["path"]).st_size,
+            signature=_f.get("signature") or blake2b_file(parent_path / _f["path"]),
+            file_desc=FileDesc[_f["desc"]],
+        )
+        for _f in files
+    }
+
 
 class StandaloneModel(Model, LocalStorageBundleMixin):
     def __init__(self, uri: URI) -> None:
@@ -183,6 +215,8 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         self.store = ModelStorage(uri)
         self.tag = StandaloneTag(uri)
         self._manifest: t.Dict[str, t.Any] = {}  # TODO: use manifest classget_conda_env
+        self.models: t.List[t.Dict[str, t.Any]] = []
+        self.sources: t.List[t.Dict[str, t.Any]] = []
         self.yaml_name = DefaultYAMLName.MODEL
         self._version = uri.object.version
 
@@ -196,12 +230,10 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         self.tag.remove(tags, ignore_errors)
 
     def _gen_steps(self, typ: str, ppl: str, workdir: Path) -> None:
-        d = self.store.snapshot_workdir / "src"
+        d = self.store.src_dir
         svc = self._get_service(ppl, workdir)
         _f = d / DEFAULT_EVALUATION_SVC_META_FNAME
-        apis = {k: v.to_yaml() for k, v in svc.apis.items()}
-        ensure_file(_f, yaml.safe_dump(apis, default_flow_style=False))
-
+        ensure_file(_f, json.dumps(svc.get_spec(), indent=4))
         if typ == EvalHandlerType.DEFAULT:
             # use default
             ppl = DEFAULT_EVALUATION_PIPELINE
@@ -214,8 +246,14 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         module, _, attr = module.partition(":")
         m = load_module(module, pkg)
         apis = dict()
+        ins: t.Any = None
         svc: t.Optional[Service] = None
 
+        # TODO: refine this ugly ad hoc
+        context_holder.context = Context(
+            pkg, version="-1", project="tmp-project-for-build"
+        )
+
         # TODO: check duplication
         for k, v in m.__dict__.items():
             if isinstance(v, Service):
@@ -224,21 +262,28 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
                 svc = v
         if attr:
             cls = getattr(m, attr)
-            # TODO: refine this ugly ad hoc
-            context_holder.context = Context(
-                Path("."), version="-1", project="tmp-project-for-build"
-            )
             ins = cls()
-            apis.update(ins.svc.apis)
+            if isinstance(ins, PipelineHandler):
+                apis.update(ins.svc.apis)
 
         from starwhale.api._impl.service import internal_api_list
 
         apis.update(internal_api_list())
 
+        # check if we need to instance the model when using custom handler
+        if ins is None:
+            for i in apis.values():
+                fn = i.func.__qualname__
+                # TODO: support deep path classes (also modules)
+                if "." in fn:
+                    ins = getattr(m, fn.split(".", 1)[0])()
+                    break
+
         if svc is None:
             svc = Service()
         for api in apis.values():
             svc.add_api_instance(api)
+        svc.api_instance = ins
         return svc
 
     @classmethod
@@ -369,18 +414,68 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         else:
             return _model_config.run.handler
 
+    def diff(self, compare_uri: URI) -> t.Dict[str, t.Any]:
+        """
+        - added: a node that exists in compare but not in base
+        - deleted: a node that not exists in compare but in base
+        - updated: a node that exists in both of base and compare but signature is different
+        - unchanged: a node that exists in both of base and compare, and signature is same
+        :param compare_uri:
+        :return: diff info
+        """
+        # TODO use remote get model info for cloud
+        if compare_uri.instance_type != InstanceType.STANDALONE:
+            raise NoSupportError(
+                f"only support standalone uri, but compare_uri({compare_uri}) is for cloud instance"
+            )
+        if self.uri.object.name != compare_uri.object.name:
+            raise NoSupportError(
+                f"only support two versions diff in one model, base model:{self.uri}, compare model:{compare_uri}"
+            )
+        _compare_model = StandaloneModel(compare_uri)
+        base_file_maps = resource_to_file_node(
+            files=self.store.manifest["resources"],
+            parent_path=self.store.snapshot_workdir,
+        )
+        compare_file_maps = resource_to_file_node(
+            files=_compare_model.store.manifest["resources"],
+            parent_path=_compare_model.store.snapshot_workdir,
+        )
+        all_paths = {
+            p for m in (base_file_maps, compare_file_maps) for p, _ in m.items()
+        }
+
+        for _p in all_paths:
+            if _p in base_file_maps and _p in compare_file_maps:
+                if base_file_maps[_p].signature == compare_file_maps[_p].signature:
+                    compare_file_maps[_p].flag = FileFlag.UNCHANGED
+                else:
+                    compare_file_maps[_p].flag = FileFlag.UPDATED
+            if _p in base_file_maps and _p not in compare_file_maps:
+                del_file = copy.copy(base_file_maps[_p])
+                del_file.flag = FileFlag.DELETED
+                compare_file_maps[_p] = del_file
+            if _p not in base_file_maps and _p in compare_file_maps:
+                compare_file_maps[_p].flag = FileFlag.ADDED
+
+        return {
+            "all_paths": all_paths,
+            "base_version": base_file_maps,
+            "compare_version": compare_file_maps,
+        }
+
     def info(self) -> t.Dict[str, t.Any]:
         _manifest = self._get_bundle_info()
         _store = self.store
         _om = {}
         if _store.snapshot_workdir.exists():
-            pth = _store.snapshot_workdir / "src" / DEFAULT_EVALUATION_JOBS_FNAME
+            pth = _store.src_dir / DEFAULT_EVALUATION_JOBS_FNAME
             if pth.exists():
                 _om = load_yaml(pth)
             else:
                 ignore_error("step_spec not found in model snapshot_workdir")
         elif _store.bundle_path.exists():
-            if tarfile.is_tarfile(_store.bundle_path):
+            if _store.bundle_path.is_file() and tarfile.is_tarfile(_store.bundle_path):
                 with TarFS(str(_store.bundle_path)) as tar:
                     with tar.open("src/" + DEFAULT_EVALUATION_JOBS_FNAME) as f:
                         _om = yaml.safe_load(f)
@@ -402,12 +497,11 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
 
         _r = []
         for _bf in self.store.iter_bundle_history():
-            if not _bf.path.is_file():
+            _manifest_path = _bf.path / DEFAULT_MANIFEST_NAME
+            if not _manifest_path.exists():
                 continue
 
-            _manifest = ModelStorage.get_manifest_by_path(
-                _bf.path, BundleType.MODEL, URIType.MODEL
-            )
+            _manifest = load_yaml(_manifest_path)
 
             _r.append(
                 dict(
@@ -450,13 +544,15 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
             bundle_type=BundleType.MODEL,
             uri_type=URIType.MODEL,
         ):
-            if not _bf.path.is_file():
+            if _bf.path.is_file():
+                # for origin swmp(tar)
+                _manifest = ModelStorage.get_manifest_by_path(
+                    _bf.path, BundleType.MODEL, URIType.MODEL
+                )
+            elif (_bf.path / DEFAULT_MANIFEST_NAME).exists():
+                _manifest = load_yaml(_bf.path / DEFAULT_MANIFEST_NAME)
+            else:
                 continue
-
-            _manifest = ModelStorage.get_manifest_by_path(
-                _bf.path, BundleType.MODEL, URIType.MODEL
-            )
-
             rs[_bf.name].append(
                 {
                     "name": _bf.name,
@@ -496,16 +592,42 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
                     workdir=workdir,
                 ),
             ),
+            (
+                self._make_meta_tar,
+                20,
+                "build model bundle",
+            ),
             (
                 self._render_manifest,
                 5,
                 "render manifest",
             ),
-            (self._make_tar, 20, "build model bundle", dict(ftype=BundleType.MODEL)),
             (self._make_auto_tags, 5, "make auto tags"),
         ]
         run_with_progress_bar("model bundle building...", operations)
 
+    def _make_meta_tar(self) -> None:
+        w = Walker(exclude=[f["name"] for f in self.models])
+        src_fs = open_fs(str(self.store.src_dir.resolve()))
+        with tarfile.open(self.store.snapshot_workdir / SWMP_SRC_FNAME, "w:") as tar:
+            for f in w.files(src_fs):
+                sub_path = f[1:]
+                tar.add(str(self.store.src_dir / sub_path), arcname=sub_path)
+                self.sources.append(
+                    {
+                        "name": os.path.basename(f),
+                        "path": f"{self.store.src_dir_name}/{sub_path}",
+                        "signature": blake2b_file(self.store.src_dir / sub_path),
+                        "duplicate_check": False,
+                        "desc": FileDesc.SRC.name,
+                        "size": file_stat(self.store.src_dir / sub_path).st_size,
+                    }
+                )
+
+    def _render_manifest(self) -> None:
+        self._manifest["resources"] = self.models + self.sources
+        super()._render_manifest()
+
     @classmethod
     def load_model_config(cls, yaml_path: Path) -> ModelConfig:
         cls._do_validate_yaml(yaml_path)
@@ -549,22 +671,32 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         _mc = model_config
 
         workdir_fs = open_fs(str(workdir.resolve()))
-        snapshot_fs = open_fs(str(self.store.snapshot_workdir.resolve()))
-        src_fs = open_fs(str(self.store.src_dir.resolve()))
-        # TODO: support glob pkg_data
-        copy_file(workdir_fs, yaml_name, snapshot_fs, DefaultYAMLName.MODEL)
+        snapshot_src_fs = open_fs(str(self.store.src_dir.resolve()))
         copy_fs(
             workdir_fs,
-            src_fs,
+            snapshot_src_fs,
             walker=self._get_src_walker(
                 workdir, _mc.run.pkg_data, _mc.run.exclude_pkg_data
             ),
             workers=DEFAULT_COPY_WORKERS,
         )
 
-        for _fname in _mc.config + _mc.model:
-            copy_file(workdir_fs, _fname, src_fs, _fname)
+        for _fname in _mc.config:
+            copy_file(workdir_fs, _fname, snapshot_src_fs, _fname)
 
+        # TODO link models to unified storage
+        for _fname in _mc.model:
+            copy_file(workdir_fs, _fname, snapshot_src_fs, _fname)
+            self.models.append(
+                {
+                    "name": os.path.basename(_fname),
+                    "path": f"{self.store.src_dir_name}/{_fname}",
+                    "signature": blake2b_file(self.store.src_dir / _fname),
+                    "duplicate_check": True,
+                    "desc": FileDesc.MODEL.name,
+                    "size": file_stat(self.store.src_dir / _fname).st_size,
+                }
+            )
         logger.info("[step:copy]finish copy files")
 
     @classmethod
@@ -579,9 +711,6 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
             if _k not in os.environ:
                 os.environ[_k] = _v
 
-    def extract(self, force: bool = False, target: t.Union[str, Path] = "") -> Path:
-        return self._do_extract(force, target)
-
     @classmethod
     def serve(
         cls,
@@ -589,11 +718,10 @@ class StandaloneModel(Model, LocalStorageBundleMixin):
         workdir: Path,
         host: str,
         port: int,
-        handlers: t.Optional[t.List[str]] = None,
     ) -> None:
         _model_config = cls.load_model_config(workdir / model_yaml)
         svc = cls._get_service(_model_config.run.handler, workdir)
-        svc.serve(host, port, handlers)
+        svc.serve(host, port)
 
 
 class CloudModel(CloudBundleModelMixin, Model):
@@ -614,3 +742,6 @@ class CloudModel(CloudBundleModelMixin, Model):
 
     def build(self, *args: t.Any, **kwargs: t.Any) -> None:
         raise NoSupportError("no support build model in the cloud instance")
+
+    def diff(self, compare_uri: URI) -> t.Dict[str, t.Any]:
+        raise NoSupportError("no support model diff in the cloud instance")
