@@ -1,13 +1,37 @@
 import os
+import sys
 import typing as t
 from pathlib import Path
 
-from starwhale.utils import console, load_yaml, pretty_bytes, in_production
-from starwhale.consts import DefaultYAMLName, DEFAULT_PAGE_IDX, DEFAULT_PAGE_SIZE
+from rich import box
+from rich.text import Text
+from rich.panel import Panel
+from rich.table import Table
+from rich.console import Group
+
+from starwhale.utils import (
+    docker,
+    console,
+    process,
+    load_yaml,
+    pretty_bytes,
+    in_production,
+)
+from starwhale.consts import (
+    FileFlag,
+    DefaultYAMLName,
+    DEFAULT_PAGE_IDX,
+    DEFAULT_PAGE_SIZE,
+    SHORT_VERSION_CNT,
+)
 from starwhale.base.uri import URI
+from starwhale.utils.fs import cmp_file_content
 from starwhale.base.type import URIType, InstanceType
 from starwhale.base.view import BaseTermView
+from starwhale.consts.env import SWEnv
+from starwhale.utils.error import FieldTypeOrValueError
 from starwhale.core.model.store import ModelStorage
+from starwhale.core.runtime.model import StandaloneRuntime
 from starwhale.core.runtime.process import Process as RuntimeProcess
 
 from .model import Model, StandaloneModel
@@ -33,6 +57,60 @@ class ModelTermView(BaseTermView):
     def info(self, fullname: bool = False) -> None:
         self._print_info(self.model.info(), fullname=fullname)
 
+    @BaseTermView._only_standalone
+    def diff(self, compare_uri: URI, show_details: bool) -> None:
+        r = self.model.diff(compare_uri)
+        text_details: t.List[Panel] = []
+        table = Table(box=box.SIMPLE, expand=False, show_lines=True)
+        table.add_column("file")
+        table.add_column("path")
+        table.add_column(
+            f"base: {self.model.version[:SHORT_VERSION_CNT]}",
+            style="magenta",
+            justify="left",
+        )
+        table.add_column(
+            f"compare: {compare_uri.object.version[:SHORT_VERSION_CNT]}",
+            style="cyan",
+            justify="left",
+        )
+        b_files = r["base_version"]
+        c_files = r["compare_version"]
+        for _p in r["all_paths"]:
+            _flag = c_files[_p].flag
+            if _flag == FileFlag.ADDED:
+                _flag_txt = f"[blue]{_flag}"
+            elif _flag == FileFlag.DELETED:
+                _flag_txt = f"[red]{_flag}"
+            elif _flag == FileFlag.UPDATED:
+                _flag_txt = f"[yellow]{_flag}"
+                if show_details:
+                    text_details.append(
+                        Panel(
+                            Text(
+                                text="".join(
+                                    cmp_file_content(
+                                        base_path=b_files[_p].path,
+                                        cmp_path=c_files[_p].path,
+                                    )
+                                )
+                            ),
+                            title=_p,
+                        )
+                    )
+            else:
+                _flag_txt = f"[green]{_flag}"
+            table.add_row(
+                os.path.basename(_p),
+                _p,
+                "existed" if _p in b_files else "-",
+                f"[green]{_flag_txt}",
+            )
+
+        console.print(Panel(table, title="diff overview"))
+        if show_details:
+            console.print(Panel(Group(*text_details), title="diff details"))
+
     @BaseTermView._pager
     @BaseTermView._header
     def history(self, fullname: bool = False) -> t.List[t.Dict[str, t.Any]]:
@@ -41,12 +119,6 @@ class ModelTermView(BaseTermView):
             title="Model History List", history=self.model.history(), fullname=fullname
         )
 
-    @BaseTermView._only_standalone
-    def extract(self, force: bool = False, target_dir: str = "") -> None:
-        console.print(":oncoming_police_car: try to extract ...")
-        path = self.model.extract(force, target_dir)
-        console.print(f":clap: extracted @ {path.resolve()} :tada:")
-
     @classmethod
     @BaseTermView._only_standalone
     def eval(
@@ -60,7 +132,38 @@ class ModelTermView(BaseTermView):
         task_index: int = 0,
         task_num: int = 0,
         runtime_uri: str = "",
+        use_docker: bool = False,
+        gencmd: bool = False,
+        image: str = "",
     ) -> None:
+        if use_docker:
+            if not runtime_uri and not image:
+                raise FieldTypeOrValueError(
+                    "runtime_uri and image both are none when use_docker"
+                )
+            if runtime_uri:
+                runtime = StandaloneRuntime(
+                    URI(runtime_uri, expected_type=URIType.RUNTIME)
+                )
+                image = runtime.store.get_docker_base_image()
+            mnt_paths = (
+                [os.path.abspath(target)]
+                if in_production() or (os.path.exists(target) and os.path.isdir(target))
+                else []
+            )
+            env_vars = {SWEnv.runtime_version: runtime_uri} if runtime_uri else {}
+            cmd = docker.gen_swcli_docker_cmd(
+                image,
+                env_vars=env_vars,
+                mnt_paths=mnt_paths,
+            )
+            console.rule(":elephant: docker cmd", align="left")
+            console.print(f"{cmd}\n")
+            if gencmd:
+                return
+            process.check_call(cmd, shell=True)
+            return
+
         kw = dict(
             project=project,
             version=version,
@@ -87,7 +190,7 @@ class ModelTermView(BaseTermView):
         else:
             _uri = URI(target, URIType.MODEL)
             _store = ModelStorage(_uri)
-            workdir = _store.loc
+            workdir = _store.src_dir
         return workdir
 
     @classmethod
@@ -159,12 +262,39 @@ class ModelTermView(BaseTermView):
         cls,
         target: str,
         model_yaml: str,
+        runtime_uri: str,
+        model_uri: str,
         host: str,
         port: int,
-        handlers: t.Optional[t.List[str]] = None,
     ) -> None:
-        workdir = cls._get_workdir(target)
-        StandaloneModel.serve(model_yaml, workdir, host, port, handlers)
+        if target and model_uri:
+            console.print("workdir and model can not both set together")
+            sys.exit(1)
+        if not target and not model_uri:
+            console.print("workdir or model needs to be set")
+            sys.exit(1)
+
+        if target:
+            workdir = cls._get_workdir(target)
+        else:
+            _m = StandaloneModel(URI(model_uri, expected_type=URIType.MODEL))
+            workdir = _m.store.src_dir
+
+        kw = dict(
+            model_yaml=model_yaml,
+            workdir=workdir,
+            host=host,
+            port=port,
+        )
+
+        if runtime_uri:
+            RuntimeProcess.from_runtime_uri(
+                uri=runtime_uri,
+                target=StandaloneModel.serve,
+                kwargs=kw,
+            ).run()
+        else:
+            StandaloneModel.serve(**kw)  # type: ignore
 
 
 class ModelTermViewRich(ModelTermView):
