@@ -121,7 +121,7 @@ _list: t.Callable[[_t_mixed_str_list], t.List[str]] = (
 
 _SUPPORT_CUDA = ["11.3", "11.4", "11.5", "11.6", "11.7"]
 _SUPPORT_CUDNN = {"8": {"support_cuda_versions": ["11.3", "11.4", "11.5", "11.6"]}}
-_SUPPORT_PYTHON_VERSIONS = ["3.7", "3.8", "3.9", "3.10"]
+_SUPPORT_PYTHON_VERSIONS = ["3.7", "3.8", "3.9", "3.10", "3.11"]
 
 
 class DockerEnv(ASDictMixin):
@@ -661,17 +661,19 @@ class Runtime(BaseBundle, metaclass=ABCMeta):
         env_name: str = "",
         env_prefix_path: str = "",
         disable_auto_inject: bool = False,
+        no_cache: bool = False,
         stdout: bool = False,
         include_editable: bool = False,
         emit_pip_options: bool = False,
         env_use_shell: bool = False,
-    ) -> None:
-        StandaloneRuntime.lock(
+    ) -> str:
+        return StandaloneRuntime.lock(
             target_dir,
             yaml_name,
             env_name,
             env_prefix_path,
             disable_auto_inject,
+            no_cache,
             stdout,
             include_editable,
             emit_pip_options,
@@ -698,6 +700,7 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
         self.tag = StandaloneTag(uri)
         self._manifest: t.Dict[str, t.Any] = {}
         self._version = uri.object.version
+        self._detected_sw_version: str = ""
 
     def info(self) -> t.Dict[str, t.Any]:
         return self._get_bundle_info()
@@ -756,6 +759,7 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
     ) -> None:
         yaml_name = kw.get("yaml_name", DefaultYAMLName.RUNTIME)
         disable_env_lock = kw.get("disable_env_lock", False)
+        no_cache = kw.get("no_cache", False)
         env_name = kw.get("env_name", "")
         env_prefix_path = kw.get("env_prefix_path", "")
         env_use_shell = kw.get("env_use_shell", False)
@@ -765,16 +769,22 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
             console.print(
                 f":alien: try to lock environment dependencies to {yaml_name}@{workdir} ..."
             )
-            self.lock(
+            content = self.lock(
                 target_dir=workdir,
                 yaml_name=yaml_name,
                 env_name=env_name,
                 env_prefix_path=env_prefix_path,
                 disable_auto_inject=False,
+                no_cache=no_cache,
                 stdout=False,
                 include_editable=include_editable,
                 env_use_shell=env_use_shell,
             )
+            # try getting starwhale version
+            for line in content.splitlines():
+                if line.startswith("starwhale=="):
+                    self._detected_sw_version = line.split("==")[1].strip()
+                    break
 
         # TODO: tune for no runtime.yaml file
         _swrt_config = self._load_runtime_config(workdir / yaml_name)
@@ -950,7 +960,7 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
         self._manifest["environment"].update(
             {
                 "lock": {
-                    "starwhale_version": STARWHALE_VERSION,
+                    "starwhale_version": self._detected_sw_version or STARWHALE_VERSION,
                     "system": platform.system(),
                     "shell": {
                         "python_env": sh_py_env,
@@ -1146,11 +1156,18 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
         python_version = get_python_version()
 
         sw_pkg = SW_PYPI_PKG_NAME
-        _swcli_version = STARWHALE_VERSION
-        if _swcli_version and _swcli_version != SW_DEV_DUMMY_VERSION:
-            sw_pkg = f"{sw_pkg}=={_swcli_version}"
+        if STARWHALE_VERSION and STARWHALE_VERSION != SW_DEV_DUMMY_VERSION:
+            sw_pkg = f"{sw_pkg}=={STARWHALE_VERSION}"
 
-        cls.render_runtime_yaml(workdir, name, mode, python_version, [sw_pkg], force)
+        cls.render_runtime_yaml(
+            workdir,
+            name,
+            mode,
+            python_version,
+            pkgs=[sw_pkg],
+            force=True,
+            auto_inject_sw=False,
+        )
 
         if not disable_create_env:
             sw_auto_dir = workdir / SW_AUTO_DIRNAME
@@ -1205,8 +1222,16 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
 
     @classmethod
     def _ensure_isolated_python_env(
-        cls, env_dir: Path, python_version: str, mode: str, invalid_rebuild: bool = True
+        cls,
+        env_dir: Path,
+        python_version: str,
+        mode: str,
+        no_cache: bool,
+        recreate_env_if_broken: bool = True,
     ) -> None:
+        if no_cache:
+            empty_dir(env_dir)
+
         if env_dir.exists():
             is_valid_conda = mode == PythonRunEnv.CONDA and check_valid_conda_prefix(
                 env_dir
@@ -1215,11 +1240,10 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
                 env_dir
             )
 
-            # TODO: add rebuild option
             if is_valid_conda or is_valid_venv:
                 return
             else:
-                if invalid_rebuild:
+                if recreate_env_if_broken:
                     empty_dir(env_dir)
                     ensure_dir(env_dir)
                 else:
@@ -1237,11 +1261,26 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
         env_name: str = "",
         env_prefix_path: str = "",
         disable_auto_inject: bool = False,
+        no_cache: bool = False,
         stdout: bool = False,
         include_editable: bool = False,
         emit_pip_options: bool = False,
         env_use_shell: bool = False,
-    ) -> None:
+    ) -> str:
+        """
+        Install dependencies and save or print lock file
+        :param target_dir: runtime work directory which contains the configured runtime.yaml
+        :param yaml_name: runtime config file name
+        :param env_name: conda environment name (used by conda env)
+        :param env_prefix_path: python env prefix path (used by both venv and conda)
+        :param disable_auto_inject: disable putting lock file info into configured runtime yaml file
+        :param no_cache: invalid all pkgs installed
+        :param stdout: just print the lock info into stdout without saving lock file
+        :param include_editable: include the editable pkg (only for venv)
+        :param emit_pip_options: use user's pip configuration when freeze pkgs (only for venv)
+        :param env_use_shell: automatically detect env in current shell session
+        :return: the lock file content
+        """
         target_dir = Path(target_dir)
         runtime_fpath = target_dir / yaml_name
         if not runtime_fpath.exists():
@@ -1258,7 +1297,6 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
                 f"env_name({env_name}), env_prefix_path({env_prefix_path}) and env_use_shell({env_use_shell}) are the mutex arguments"
             )
 
-        prefix_path = ""
         if env_name:
             if mode == PythonRunEnv.VENV:
                 raise NoSupportError(
@@ -1271,7 +1309,9 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
             prefix_path = env_prefix_path
         else:
             _sw_auto_path = target_dir / SW_AUTO_DIRNAME / mode
-            cls._ensure_isolated_python_env(_sw_auto_path, expected_pyver, mode)
+            cls._ensure_isolated_python_env(
+                _sw_auto_path, expected_pyver, mode, no_cache
+            )
             prefix_path = str(_sw_auto_path)
 
         cls._install_dependencies_with_runtime_yaml(
@@ -1308,10 +1348,12 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
                 f"{mode}: expected python({expected_pyver}) is not equal to detected python({detected_pyver})"
             )
 
+        with open(temp_lock_path) as f:
+            content = f.read()
+
         if stdout:
             console.rule("dependencies lock")
-            with open(temp_lock_path) as f:
-                console.print(f.read())
+            console.print(content)
             os.unlink(temp_lock_path)
         else:
             dest_fname = (
@@ -1325,6 +1367,8 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
             if not disable_auto_inject and runtime_fpath.exists():
                 cls._update_runtime_dep_lock_field(runtime_fpath, dest_fname)
 
+        return content
+
     @staticmethod
     def _update_runtime_dep_lock_field(runtime_fpath: Path, lock_fname: str) -> None:
         content = load_yaml(runtime_fpath)
@@ -1340,30 +1384,48 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
 
     @staticmethod
     def render_runtime_yaml(
-        workdir: Path,
+        workdir: t.Union[Path, str],
         name: str,
         mode: str,
-        python_version: str,
-        pkgs: t.List[str],
+        python_version: str = "",
+        pkgs: t.Optional[t.List[str]] = None,
         force: bool = False,
+        auto_inject_sw: bool = True,
+        cuda_version: t.Optional[str] = None,
     ) -> None:
+        workdir = Path(workdir)
         _rm = workdir / DefaultYAMLName.RUNTIME
 
+        if python_version == "":
+            python_version = get_python_version()
+
         if _rm.exists() and not force:
             raise ExistedError(f"{_rm} was already existed")
 
         if mode == PythonRunEnv.CONDA:
-            lock_file = RuntimeLockFileType.CONDA
+            lock_fname = RuntimeLockFileType.CONDA
             lock_content = f"name: {name}"
         else:
-            lock_file = RuntimeLockFileType.VENV
+            lock_fname = RuntimeLockFileType.VENV
             lock_content = ""
 
-        if not Path(lock_file).exists():
-            ensure_file(lock_file, content=lock_content)
+        lock_fpath = workdir / lock_fname
+        if not lock_fpath.exists():
+            ensure_file(lock_fpath, content=lock_content)
+
+        pkgs = pkgs or []
+        if auto_inject_sw and not any(
+            [p.strip().startswith(SW_PYPI_PKG_NAME) for p in pkgs]
+        ):
+            sw_pkg = SW_PYPI_PKG_NAME
+            if STARWHALE_VERSION and STARWHALE_VERSION != SW_DEV_DUMMY_VERSION:
+                sw_pkg = f"{sw_pkg}=={STARWHALE_VERSION}"
+            pkgs.append(sw_pkg)
+
+        pkgs = [p.strip() for p in pkgs if p.strip()]
 
         ensure_dir(workdir)
-        config = dict(
+        config: t.Dict[str, t.Any] = dict(
             name=name,
             mode=mode,
             environment={
@@ -1372,11 +1434,14 @@ class StandaloneRuntime(Runtime, LocalStorageBundleMixin):
                 "os": SupportOS.UBUNTU,
             },
             dependencies=[
-                lock_file,
+                lock_fname,
                 {"pip": pkgs},
             ],
             api_version=RUNTIME_API_VERSION,
         )
+        if cuda_version is not None:
+            config["environment"]["cuda"] = cuda_version
+
         ensure_file(_rm, yaml.safe_dump(config, default_flow_style=False))
 
     def dockerize(
