@@ -2,6 +2,7 @@ import os
 import errno
 import shutil
 import typing as t
+import difflib
 import hashlib
 import tarfile
 from enum import IntEnum
@@ -45,7 +46,11 @@ def ensure_file(
     os.chmod(path, mode)
 
 
-def empty_dir(p: t.Union[str, Path]) -> None:
+def empty_dir(
+    p: t.Union[str, Path],
+    ignore_errors: bool = False,
+    onerror: t.Optional[t.Callable] = None,
+) -> None:
     if not p:
         return
 
@@ -55,7 +60,7 @@ def empty_dir(p: t.Union[str, Path]) -> None:
 
     def _self_empty() -> None:
         if path.is_dir():
-            shutil.rmtree(str(path.resolve()))
+            shutil.rmtree(str(path.resolve()), ignore_errors, onerror)
         else:
             path.unlink()
 
@@ -117,6 +122,31 @@ def blake2b_file(fpath: t.Union[str, Path]) -> str:
     return _hash.hexdigest()
 
 
+def cmp_file_content(
+    base_path: t.Union[str, Path], cmp_path: t.Union[str, Path]
+) -> t.List[str]:
+    base_path = Path(base_path)
+    cmp_path = Path(cmp_path)
+    res = []
+    with base_path.open("r") as base, cmp_path.open("r") as cmp:
+        diff = difflib.unified_diff(
+            base.readlines(),
+            cmp.readlines(),
+            fromfile=base_path.name,
+            tofile=cmp_path.name,
+            n=0,
+        )
+        for line in diff:
+
+            def skip(content: str) -> bool:
+                prefixes = ["---", "+++", "@@", " "]
+                return not any([content.startswith(prefix) for prefix in prefixes])
+
+            if skip(content=line):
+                res.append(line)
+    return res
+
+
 def get_path_created_time(p: Path) -> str:
     created_at = os.path.getctime(str(p.absolute()))
     return timestamp_to_datatimestr(created_at)
@@ -201,3 +231,9 @@ def is_within_dir(parent: t.Union[str, Path], child: t.Union[str, Path]) -> bool
 
     prefix = os.path.commonprefix([abs_parent, abs_child])
     return prefix == abs_parent
+
+
+def file_stat(path: t.Union[str, Path]) -> os.stat_result:
+    path = str(path)
+    abs_path = os.path.abspath(path)
+    return Path(abs_path).stat()
