@@ -6,7 +6,7 @@ import logging
 import loguru
 
 
-class StreamWrapper(object):
+class StreamWrapper:
     is_wrapper = True
 
     def __init__(
@@ -25,7 +25,7 @@ class StreamWrapper(object):
         # TODO: splitlines?
         self.logger.opt(depth=1).log(self.level, s.rstrip())
 
-    def __getatt__(self, name: str) -> t.Any:
+    def __getattr__(self, name: str) -> t.Any:
         return getattr(self.stream, name)
 
     def flush(self) -> None:
